@@ -16,9 +16,9 @@ from __future__ import absolute_import, with_statement
 import code
 import collections
 import contextlib
+import logging
 import socket
 import sys
-import traceback
 try:
     from cStringIO import StringIO
 except ImportError:
@@ -30,6 +30,8 @@ from . import io, scheduler
 __all__ = ["run_backdoor", "backdoor_handler"]
 
 
+log = logging.getLogger("greenhouse.backdoor")
+
 PREAMBLE = "Python %s on %s" % (sys.version, sys.platform)
 PS1 = getattr(sys, "ps1", ">>> ")
 PS2 = getattr(sys, "ps2", "... ")
@@ -53,6 +55,7 @@ def run_backdoor(address, namespace=None):
         connection create its own empty namespace
     :type namespace: dict or None
     """
+    log.info("starting on %r" % (address,))
     serversock = io.Socket()
     serversock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
     serversock.bind(address)
@@ -60,6 +63,7 @@ def run_backdoor(address, namespace=None):
 
     while 1:
         clientsock, address = serversock.accept()
+        log.info("connection received from %r" % (address,))
         scheduler.schedule(backdoor_handler, args=(clientsock, namespace))
 
 
@@ -83,7 +87,7 @@ def backdoor_handler(clientsock, namespace=None):
 
     clientsock.sendall(PREAMBLE + "\n" + PS1)
 
-    for input_line in _LineProducer(clientsock):
+    for input_line in _produce_lines(clientsock):
         input_line = input_line.rstrip()
         if input_line:
             input_line = '\n' + input_line
@@ -110,51 +114,59 @@ def backdoor_handler(clientsock, namespace=None):
 
 @contextlib.contextmanager
 def _wrap_stdio(stdout, stderr):
+    @scheduler.local_incoming_hook
+    @scheduler.local_outgoing_hook
+    def hook(direction, coro):
+        if direction == 1:
+            # incoming
+            sys.stdout = stdout
+            sys.stderr = stderr
+        else:
+            # outgoing
+            sys.stdout = real_stdout
+            sys.stderr = real_stderr
+
+    real_stdout = sys.stdout
+    real_stderr = sys.stderr
+
     stdout.seek(0)
     stderr.seek(0)
     stdout.truncate()
     stderr.truncate()
 
-    real_stdout = sys.stdout
-    real_stderr = sys.stderr
-
-    sys.stdout = stdout
-    sys.stderr = stderr
+    hook(1, None)
 
     yield
 
-    sys.stdout = real_stdout
-    sys.stderr = real_stderr
+    hook(2, None)
+    scheduler.remove_local_incoming_hook(hook)
+    scheduler.remove_local_outgoing_hook(hook)
 
 
-class _LineProducer(object):
-    def __init__(self, sock):
-        self.sock = sock
-        self.buf = ""
-        self.lines = collections.deque()
+# yields lines and closes connection on '\x04' (sent by telnet on Ctrl-D)
+def _produce_lines(sock):
+    lines = collections.deque()
+    buf = ""
 
-    def readline(self):
-        if self.lines:
-            return self.lines.popleft()
+    while 1:
+        if lines:
+            yield lines.popleft()
 
-        while not self.lines:
-            block = self.sock.recv(8192)
+        while not lines:
+            block = sock.recv(8192)
             if not block:
-                return None
+                if buf:
+                    yield buf
+                log.info("EOF received, closing connection")
+                return
 
-            if block == '\x04':
-                self.sock.close()
-                return None
+            if block[0] == '\x04':
+                sock.close()
+                if buf:
+                    yield buf
+                log.info("Ctrl-D received, closing connection")
+                return
 
-            self.buf += block
-            lines = self.buf.split('\n')
-            self.lines.extend(lines[:-1])
-            self.buf = lines[-1]
-
-        return self.lines.popleft()
-
-    def __iter__(self):
-        line = self.readline()
-        while line is not None:
-            yield line
-            line = self.readline()
+            l = (buf + block).split("\n")
+            lines.extend(l[:-1])
+            buf = l[-1]
