@@ -145,7 +145,8 @@ def green_popen(cmd, mode='r', bufsize=-1):
     kwarg = {pipe: green_subprocess.PIPE}
 
     proc = green_subprocess.Popen(cmd, shell=1, bufsize=bufsize, **kwarg)
-    return green_popen_pipe.fromfd(getattr(proc, pipe), mode, proc.pid)
+    return green_popen_pipe.fromfd(
+            getattr(proc, pipe).fileno(), mode, proc.pid)
 
 @functools.wraps(original_os_popen2)
 def green_popen2(cmd, mode='r', bufsize=-1):
@@ -183,6 +184,7 @@ def green_spawner(func):
         if details & 0xff:
             return -(details & 0xff)
         return details >> 8
+    return green_version
 
 green_spawnl = green_spawner(original_os_spawnl)
 green_spawnle = green_spawner(original_os_spawnle)
