@@ -6,6 +6,8 @@ import operator
 import select
 import sys
 
+_original_select = select.select
+
 
 class Poll(object):
     "a greenhouse poller using the poll system call"
@@ -178,8 +180,12 @@ class Select(object):
         self._registry.get(fd, {}).pop(counter, None)
 
         # rewrite the full mask
-        self._currentmasks[fd] = reduce(
+        newmask = reduce(
                 operator.or_, self._registry[fd].itervalues(), 0)
+        if newmask:
+            self._currentmasks[fd] = newmask
+        else:
+            self._currentmasks.pop(fd, 0)
 
         if not self._registry[fd]:
             self._registry.pop(fd)
@@ -193,7 +199,7 @@ class Select(object):
                 wlist.append(fd)
             if eventmask & self.ERRMASK:
                 xlist.append(fd)
-        rlist, wlist, xlist = select.select(rlist, wlist, xlist, timeout)
+        rlist, wlist, xlist = _original_select(rlist, wlist, xlist, timeout)
         events = collections.defaultdict(int)
         for fd in rlist:
             events[fd] |= self.INMASK
