@@ -1,4 +1,3 @@
-import bisect
 import collections
 import functools
 import heapq
@@ -74,8 +73,10 @@ class Event(object):
     def wait(self, timeout=None):
         """pause the current coroutine until this event is set
 
-        if :meth:`set` method has been called, this method will not block at
-        all. otherwise it will block until :meth:`set` method is called
+        .. note::
+
+            this method will block the current coroutine if :meth:`set` has not
+            been called.
 
         :param timeout:
             the maximum amount of time to block in seconds. the default of
@@ -98,10 +99,9 @@ class Event(object):
         scheduler.state.mainloop.switch()
 
         if timeout is not None:
-            timedout = not scheduler._remove_from_timedout(waketime, current)
-            if timedout:
+            if not scheduler._remove_timer(waketime, current):
                 self._waiters.remove(current)
-            return timedout
+                return True
 
         return False
 
@@ -128,9 +128,16 @@ class Lock(object):
         """
         return self._locked
 
+    locked_lock = locked
+
     def acquire(self, blocking=True):
         """lock the lock, blocking until it becomes available
 
+        .. note::
+
+            this method will block the current coroutine if the lock is not
+            already owned.
+
         :param blocking:
             whether to block if the lock is already owned (default ``True``)
         :type blocking: bool
@@ -155,6 +162,8 @@ class Lock(object):
             self._owner = current
         return True
 
+    acquire_lock = acquire
+
     def release(self):
         """open the lock back up to wake up a waiting greenlet
 
@@ -173,6 +182,8 @@ class Lock(object):
             self._locked = False
             self._owner = None
 
+    release_lock = release
+
     def __enter__(self):
         return self.acquire()
 
@@ -195,6 +206,11 @@ class RLock(Lock):
         gets incremented. if it is owned by a different greenlet then it will
         block until the lock becomes available.
 
+        .. note::
+
+            this method will block the current coroutine if the lock is not
+            already owned by another coroutine.
+
         :param blocking:
             whether to block if the lock is owned by a different greenlet
             (default ``True``)
@@ -273,6 +289,11 @@ class Condition(object):
     def wait(self, timeout=None):
         """wait to be woken up by the condition
 
+        .. note::
+
+            this method will block the current coroutine until a :meth:`notify`
+            wakes it back up.
+
         :raises:
             `RuntimeError` if the underlying lock hasn't been
             :meth:`acquired <Lock.acquire>`
@@ -292,7 +313,7 @@ class Condition(object):
         self._lock.acquire()
 
         if timeout is not None:
-            timedout = not scheduler._remove_from_timedout(waketime, current)
+            timedout = not scheduler._remove_timer(waketime, current)
             if timedout:
                 self._waiters.remove((current, waketime))
             return timedout
@@ -342,7 +363,12 @@ class Semaphore(object):
         self._waiters = collections.deque()
 
     def acquire(self, blocking=True):
-        """decrement the counter, blocking if it is already at 0
+        """decrement the counter, waiting if it is already at 0
+
+        .. note::
+
+            if the counter is already at 0, this method will block the current
+            coroutine until a :meth:`release` increments it again.
 
         :param blocking:
             whether or not to block if the counter is already at 0 (default
@@ -392,6 +418,11 @@ class BoundedSemaphore(Semaphore):
     def release(self):
         """increment the counter, waking up a waiter if there was any
 
+        .. note::
+
+            if the counter is already at 0, this method will block the current
+            coroutine until a :meth:`release` increments it again.
+
         :raises: `ValueError` if this increment would have crossed the maximum
         """
         if self._value >= self._initial_value:
@@ -409,8 +440,10 @@ class Local(object):
     """
     _main_standin = type('', (), {})()
 
-    def __init__(self):
-        object.__setattr__(self, "_local_data", weakref.WeakKeyDictionary())
+    def __new__(cls):
+        obj = object.__new__(cls)
+        object.__setattr__(obj, "_local_data", weakref.WeakKeyDictionary())
+        return obj
 
     def __getattr__(self, name):
         current = compat.getcurrent()
@@ -468,23 +501,26 @@ class Thread(object):
         self._started = False
         self._finished = Event()
         self._glet = None
+        self._ident = None
         # verbose is ignored
 
     def __repr__(self):
         status = "initial"
         if self._started:
             status = "started"
-        if not self._finished.is_set():
+        if self._finished.is_set():
             status = "stopped"
         return "<%s (%s, %s)>" % (type(self).__name__, self.name, status)
 
     def _activate(self):
         self._started = True
         type(self)._active[self._glet] = self
+        type(self)._active_by_id[self.ident] = self
 
     def _deactivate(self):
         self._finished.set()
         type(self)._active.pop(self._glet)
+        type(self)._active_by_id.pop(self.ident)
 
     def start(self):
         """schedule to start the greenlet that runs this thread's function
@@ -496,9 +532,13 @@ class Thread(object):
         def run():
             try:
                 self.run(*self._args, **self._kwargs)
+            except SystemExit:
+                # only shut down the thread, not the whole process
+                pass
             finally:
                 self._deactivate()
         self._glet = scheduler.greenlet(run)
+        self._ident = id(self._glet)
         scheduler.schedule(self._glet)
         self._activate()
 
@@ -513,6 +553,11 @@ class Thread(object):
     def join(self, timeout=None):
         """block until this thread terminates
 
+        .. note::
+
+            this method can block the calling coroutine if the thread has not
+            yet completed.
+
         :param timeout:
             the maximum time to wait. with the default of ``None``, waits
             indefinitely
@@ -531,7 +576,7 @@ class Thread(object):
     @property
     def ident(self):
         "unique identifier for this thread"
-        return id(self._glet) if self._glet is not None else None
+        return self._ident
 
     def is_alive(self):
         """indicates whether the thread is currently running
@@ -560,7 +605,7 @@ class Thread(object):
         :param daemonic: whether attempting to turn daemon mode on or off
         :type daemonic: bool
         """
-        return
+        pass
     setDaemon = set_daemon
 
     daemon = property(is_daemon, set_daemon,
@@ -587,45 +632,12 @@ class Thread(object):
     _counter = 0
 
     @classmethod
-    def _newname(cls):
+    def _newname(cls, template="Thread-%d"):
         cls._counter += 1
-        return "Thread-%d" % cls._counter
+        return template % cls._counter
 
     _active = {}
-
-_main_thread = object.__new__(Thread)
-_main_thread.__dict__.update({
-    'name': 'MainThread',
-    '_target': None,
-    '_args': (),
-    '_kwargs': {},
-    '_started': True,
-    '_finished': Event(),
-    '_glet': compat.main_greenlet,
-})
-
-_dummy_thread = object.__new__(Thread)
-_dummy_thread.__dict__.update({
-    'name': 'GreenThread',
-    '_target': None,
-    '_args': (),
-    '_kwargs': {},
-    '_started': True,
-    '_finished': Event(),
-    '_glet': None,
-})
-
-
-def _enumerate_threads():
-    return Thread._active.values()
-
-def _active_thread_count():
-    return len(Thread._active)
-
-def _current_thread():
-    if compat.getcurrent() is compat.main_greenlet:
-        return _main_thread
-    return Thread._active.get(compat.getcurrent(), _dummy_thread)
+    _active_by_id = {}
 
 
 class Timer(Thread):
@@ -669,7 +681,10 @@ class Timer(Thread):
     def wrap(cls, secs, args=(), kwargs=None):
         """a classmethod decorator to immediately turn a function into a timer
 
-        you won't find this on `threading.Timer`, it is an extension to that API
+        .. note::
+
+            you won't find this on `threading.Timer`, it is an extension to
+            that API
 
         this is a function *returning a decorator*, so it is used like so:
 
@@ -736,8 +751,11 @@ class Queue(object):
     def get(self, blocking=True, timeout=None):
         """get an item out of the queue
 
-        this method will block if `blocking` is ``True`` (default) and the
-        queue is :meth:`empty`
+        .. note::
+
+            if `blocking` is ``True`` (the default) and the queue is
+            :meth`empty`, this method will block the current coroutine until
+            something has been :meth:`put`.
 
         :param blocking:
             whether to block if there is no data yet available (default
@@ -769,7 +787,7 @@ class Queue(object):
             scheduler.state.mainloop.switch()
 
             if timeout is not None:
-                if not scheduler._remove_from_timedout(waketime, current):
+                if not scheduler._remove_timer(waketime, current):
                     self._waiters.remove((current, waketime))
                     raise Empty()
 
@@ -793,9 +811,11 @@ class Queue(object):
     def put(self, item, blocking=True, timeout=None):
         """put an item into the queue
 
-        if the queue was instantiated with a nonzero `maxsize` and that size
-        has already been reached, this method will block until another greenlet
-        :meth:`get`\ s an item out
+        .. note::
+
+            if the queue was created with a `maxsize` and it is currently
+            :meth:`full`, this method will block the calling coroutine until
+            another coroutine :meth:`get`\ s an item.
 
         :param item: the object to put into the queue, can be any type
         :param blocking:
@@ -826,7 +846,7 @@ class Queue(object):
             scheduler.state.mainloop.switch()
 
             if timeout is not None:
-                if not scheduler._remove_from_timedout(waketime, current):
+                if not scheduler._remove_timer(waketime, current):
                     self._waiters.remove((current, waketime))
                     raise Full()
 
@@ -871,8 +891,10 @@ class Queue(object):
     def join(self, timeout=None):
         """wait for task completions
 
-        blocks until either :meth:`task_done` has been called for every
-        :meth:`put` call
+        .. note::
+
+            if :meth:`task_done` has not been called for every :meth:`put`
+            call, this method will block until it has.
 
         the queue is still usable after a :meth:`join` call. a return from
         :meth:`join` simply indicates that the queue has no jobs `currently`
@@ -918,7 +940,7 @@ class PriorityQueue(Queue):
         never block
     :type maxsize: int
 
-    mirrors the standard library `Queue.LifoQueue` API
+    mirrors the standard library `Queue.PriorityQueue` API
     """
     _data_type = list
 
